==> lkrg-0.5.tar.gz: lkrg-0.5/src/modules/exploit_detection/syscalls/keyring/p_key_change_session_keyring/p_key_change_session_keyring.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept key_change_session_keyring function
 *
 * Notes:
 *  - Exploit Detection features needs to whitelist process which
 *    legitimately increases or changes their privileges structure.
 *
 * Caveats:
 *  - Keyring support
 *
 * Timeline:
 *  - Created: 12.XII.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../../p_lkrg_main.h"


char p_key_change_session_keyring_kretprobe_state = 0x0;

static struct kretprobe p_key_change_session_keyring_kretprobe = {
    .kp.symbol_name = "key_change_session_keyring",
    .handler = p_key_change_session_keyring_ret,
    .entry_handler = p_key_change_session_keyring_entry,
    .data_size = sizeof(struct p_key_change_session_keyring_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_key_change_session_keyring_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_key_change_session_keyring_entry>\n");

   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_debug_kprobe_log(
          "Leaving function <p_key_change_session_keyring_entry>\n");

   return 0x0;
}


int p_key_change_session_keyring_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;

   p_debug_kprobe_log(
          "Entering function <p_key_change_session_keyring_ret>\n");
   p_debug_kprobe_log(
          "capset returned value => %ld comm[%s] Pid:%d parent[%d]\n",
                       p_regs->ax,current->comm,current->pid,current->real_parent->pid);

   // Update process
   spin_lock(&p_rb_ed_pids_lock);
   if ( (p_tmp = p_rb_find_ed_pid(&p_global_ed_pids_root, task_pid_nr(current))) != NULL) {
      // Always update information - intercepted void function
      // This process is on the ED list - update information!
      p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",current->pid);
      p_update_ed_process(p_tmp, current);
      p_set_ed_process_on(p_tmp);
   }
   spin_unlock(&p_rb_ed_pids_lock);

   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_key_change_session_keyring_ret>\n");

   return 0x0;
}


int p_install_key_change_session_keyring_hook(void) {

   int p_ret;

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_install_key_change_session_keyring_hook>\n");

   if ( (p_ret = register_kretprobe(&p_key_change_session_keyring_kretprobe)) < 0) {
      p_print_log(P_LKRG_ERR, "[kretprobe] register_kretprobe() for <%s> failed! [err=%d]\n",
                  p_key_change_session_keyring_kretprobe.kp.symbol_name,
                  p_ret);
      goto p_install_key_change_session_keyring_hook_out;
   }
   p_print_log(P_LKRG_INFO, "Planted [kretprobe] <%s> at: %p\n",
               p_key_change_session_keyring_kretprobe.kp.symbol_name,
               p_key_change_session_keyring_kretprobe.kp.addr);
   p_key_change_session_keyring_kretprobe_state = 0x1;

//   p_ret = 0x0; <- should be 0x0 anyway...

p_install_key_change_session_keyring_hook_out:

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_install_key_change_session_keyring_hook> (p_ret => %d)\n",p_ret);

   return p_ret;
}


void p_uninstall_key_change_session_keyring_hook(void) {

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Entering function <p_uninstall_key_change_session_keyring_hook>\n");

   if (!p_key_change_session_keyring_kretprobe_state) {
      p_print_log(P_LKRG_INFO, "[kretprobe] <%s> at 0x%p is NOT installed\n",
                  p_key_change_session_keyring_kretprobe.kp.symbol_name,
                  p_key_change_session_keyring_kretprobe.kp.addr);
   } else {
      unregister_kretprobe(&p_key_change_session_keyring_kretprobe);
      p_print_log(P_LKRG_INFO, "Removing [kretprobe] <%s> at 0x%p nmissed[%d]\n",
                  p_key_change_session_keyring_kretprobe.kp.symbol_name,
                  p_key_change_session_keyring_kretprobe.kp.addr,
                  p_key_change_session_keyring_kretprobe.nmissed);
      p_key_change_session_keyring_kretprobe_state = 0x0;
   }

// STRONG_DEBUG
   p_debug_log(P_LKRG_STRONG_DBG,
          "Leaving function <p_uninstall_key_change_session_keyring_hook>\n");
}

==> lkrg-0.8.tar.gz: lkrg-0.8/src/modules/exploit_detection/syscalls/keyring/p_key_change_session_keyring/p_key_change_session_keyring.c <==
/*
 * pi3's Linux kernel Runtime Guard
 *
 * Component:
 *  - Intercept key_change_session_keyring function
 *
 * Notes:
 *  - None
 *
 * Caveats:
 *  - Keyring support
 *
 * Timeline:
 *  - Created: 12.XII.2017
 *
 * Author:
 *  - Adam 'pi3' Zabrocki (http://pi3.com.pl)
 *
 */

#include "../../../../../p_lkrg_main.h"


char p_key_change_session_keyring_kretprobe_state = 0x0;

static struct kretprobe p_key_change_session_keyring_kretprobe = {
    .kp.symbol_name = "key_change_session_keyring",
    .handler = p_key_change_session_keyring_ret,
    .entry_handler = p_key_change_session_keyring_entry,
    .data_size = sizeof(struct p_key_change_session_keyring_data),
    /* Probe up to 40 instances concurrently. */
    .maxactive = 40,
};


int p_key_change_session_keyring_entry(struct kretprobe_instance *p_ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_key_change_session_keyring_entry>\n");

   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // This process is on the ED list - set temporary 'disable' flag!
      p_set_ed_process_off(p_tmp);
   }
   p_tasks_write_unlock(&p_flags);

   p_debug_kprobe_log(
          "Leaving function <p_key_change_session_keyring_entry>\n");

   return 0x0;
}


int p_key_change_session_keyring_ret(struct kretprobe_instance *ri, struct pt_regs *p_regs) {

   struct p_ed_process *p_tmp;
   unsigned long p_flags;

   p_debug_kprobe_log(
          "Entering function <p_key_change_session_keyring_ret>\n");
   p_debug_kprobe_log(
          "capset returned value => %ld comm[%s] Pid:%d parent[%d]\n",
                       p_regs_get_ret(p_regs),current->comm,current->pid,current->real_parent->pid);

   // Update process
   p_tasks_write_lock(&p_flags);
   if ( (p_tmp = p_find_ed_by_pid(task_pid_nr(current))) != NULL) {
      // Always update information - intercepted void function
      // This process is on the ED list - update information!
      p_print_log(P_LKRG_INFO, "Updating ED pid[%d]\n",current->pid);
      p_update_ed_process(p_tmp, current, 0);
      p_set_ed_process_on(p_tmp);
   }
   p_tasks_write_unlock(&p_flags);

   p_ed_enforce_validation();

   p_debug_kprobe_log(
          "Leaving function <p_key_change_session_keyring_ret>\n");

   return 0x0;
}


GENERATE_INSTALL_FUNC(key_change_session_keyring)
